import sys
import os
import json
import urllib.parse

import urllib.request
import xbmcplugin
import xbmcgui
import xbmcvfs
import xbmc
from resources.lib import webshare_api


handle = int(sys.argv[1])
CREDENTIALS_FILE = os.path.join(xbmcvfs.translatePath("special://profile/addon_data/plugin.video.webshare/"), "credentials.json")
token = None
def log(message):
    xbmc.log(f"[Webshare] {message}", level=xbmc.LOGINFO)

def save(token,u,p):
    os.makedirs(os.path.dirname(CREDENTIALS_FILE), exist_ok=True)
    with open(CREDENTIALS_FILE, 'w') as f:
        json.dump({'token': token,'u':u,'p':p}, f)

def load_token():
    if os.path.exists(CREDENTIALS_FILE):
        with open(CREDENTIALS_FILE, 'r') as f:
            data = json.load(f)
            return data.get('token')
    return None

def load_u():
    if os.path.exists(CREDENTIALS_FILE):
        with open(CREDENTIALS_FILE, 'r') as f:
            data = json.load(f)
            return data.get('u')
    return None

def load_p():
    if os.path.exists(CREDENTIALS_FILE):
        with open(CREDENTIALS_FILE, 'r') as f:
            data = json.load(f)
            return data.get('p')
    return None

def prompt_login():
    username = xbmcgui.Dialog().input("Enter Webshare Username", type=xbmcgui.INPUT_ALPHANUM)
    if not username:
        return
    password = xbmcgui.Dialog().input("Enter Webshare Password", type=xbmcgui.INPUT_ALPHANUM, option=xbmcgui.ALPHANUM_HIDE_INPUT)
    if not password:
        return

    try:
        new_token = webshare_api.login(username, password)
        save(new_token,username,password)
        xbmcgui.Dialog().ok("Login", "Token obtained and saved.")
    except Exception as e:
        xbmcgui.Dialog().notification("Login Failed", str(e), xbmcgui.NOTIFICATION_ERROR)


def list_search_results(query):
    results = webshare_api.search(query, sort="recent", limit=50, offset=0, category="video")
    for item in results:
        url = f"{sys.argv[0]}?action=play&id={item['id']}"
        list_item = xbmcgui.ListItem(label=item['name'])
        info = list_item.getVideoInfoTag()
        info.setTitle(item['name'])
        if item['img']:
            list_item.setArt({'thumb': item['img'], 'icon': item['img']})
        list_item.setPath(url)  # ← THIS is required
        xbmcplugin.addDirectoryItem(handle=handle, url=url, listitem=list_item, isFolder=False)
    xbmcplugin.endOfDirectory(handle)


def list_series_results():
    handle = int(sys.argv[1])
    url = "https://repo.meteolab.online/api/series"

    with urllib.request.urlopen(url) as response:
        raw = response.read().decode("utf-8")
        results = json.loads(raw)

    for name in results:
        item_url = f"{sys.argv[0]}?action=list_seasons&serie={urllib.parse.quote(name)}"
        list_item = xbmcgui.ListItem(label=name)
        list_item.setPath(item_url)
        xbmcplugin.addDirectoryItem(handle=handle, url=item_url, listitem=list_item, isFolder=True)

    xbmcplugin.endOfDirectory(handle)

def list_seasons(serie):
    url = f"https://repo.meteolab.online/api/series/{urllib.parse.quote(serie)}"
    log(f"Fetching seasons from {url}")
    try:
        with urllib.request.urlopen(url) as response:
            data = json.loads(response.read().decode("utf-8"))
    except Exception as e:
        log(f"Failed to fetch seasons: {e}")
        xbmcplugin.endOfDirectory(handle, succeeded=False)
        return

    for season in data:
        log(f"Adding season: {season}")
        item_url = f"{sys.argv[0]}?action=list_episodes&serie={urllib.parse.quote(serie)}&season={urllib.parse.quote(season)}"
        item = xbmcgui.ListItem(label=f"Season {season}")
        item.setPath(item_url)
        xbmcplugin.addDirectoryItem(handle=handle, url=item_url, listitem=item, isFolder=True)

    xbmcplugin.endOfDirectory(handle)

def list_episodes(serie, season):
    url = f"https://repo.meteolab.online/api/series/{urllib.parse.quote(serie)}/{urllib.parse.quote(season)}"
    log(f"Fetching episodes from {url}")
    try:
        with urllib.request.urlopen(url) as response:
            data = json.loads(response.read().decode("utf-8"))
    except Exception as e:
        log(f"Failed to fetch episodes: {e}")
        xbmcplugin.endOfDirectory(handle, succeeded=False)
        return

    for episode in data:
        label = f"S{season}E{episode}"
        log(f"Adding episode: {label}")
        item_url = f"{sys.argv[0]}?action=show_episode&serie={urllib.parse.quote(serie)}&season={urllib.parse.quote(season)}&episode={urllib.parse.quote(episode)}"
        item = xbmcgui.ListItem(label=label)
        item.setPath(item_url)
        xbmcplugin.addDirectoryItem(handle=handle, url=item_url, listitem=item, isFolder=True)

    xbmcplugin.endOfDirectory(handle)

def show_episode(serie, season, episode):
    url = f"https://repo.meteolab.online/api/series/{urllib.parse.quote(serie)}/{urllib.parse.quote(season)}/{urllib.parse.quote(episode)}"
    log(f"Fetching episode file list from {url}")
    try:
        with urllib.request.urlopen(url) as response:
            lines = response.read().decode("utf-8").splitlines()
            data = [json.loads(line) for line in lines]
    except Exception as e:
        log(f"Failed to fetch episode: {e}")
        xbmcplugin.endOfDirectory(handle, succeeded=False)
        return

    for file in data:
        label = file.get("name", file["ident"])
        log(f"Adding file: {label}")
        item_url = f"{sys.argv[0]}?action=play&id={file['ident']}"
        item = xbmcgui.ListItem(label=label)
        item.setPath(item_url)
        xbmcplugin.addDirectoryItem(handle=handle, url=item_url, listitem=item, isFolder=False)

    xbmcplugin.endOfDirectory(handle)

def perform_login_from_stored_credentials():
    global token
    if token:
        return
    token = load_token()
    if token:
        log(f"Loaded token from disk: {token}")
    else:
        log("No token found")

# perform_login_from_stored_credentials()

def play_video(handle, params):
    try:
        # Retrieve the URL from the API
        t = webshare_api.login(load_u(),load_p())
        link = webshare_api.load_item_link(ident=params['id'], token=t)
        log(f"Play URL: {link}")

        # Validate the URL
        if not link or not link.startswith(('http://', 'https://')):
            raise Exception("Invalid or missing URL")

        # Create a ListItem for playback
        list_item = xbmcgui.ListItem(path=link)
        list_item.setProperty("IsPlayable", "true")

        # Use InfoTagVideo for modern Kodi compatibility
        info_tag = list_item.getVideoInfoTag()
        info_tag.setMediaType('video')

        # Set MIME type (default to video/avi for .avi files)
        # mime_type = 'video/avi'  # Adjust if needed (e.g., video/mp4 for MP4)
        # list_item.setMimeType(mime_type)
        # log(f"Set MIME type: {mime_type}")

        # Add headers to the URL
        headers = {
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36',
            'Referer': 'https://vip.17.dl.wsfiles.cz/',
            'Accept': '*/*',
            'Connection': 'keep-alive',
        }
        header_string = '|'.join(f"{key}={urllib.parse.quote(value)}" for key, value in headers.items())
        final_url = f"{link}|{header_string}"
        list_item.setPath(final_url)
        log(f"Final playback URL: {final_url}")

        xbmc.Player().play(final_url)
        log("Playback triggered")

    except Exception as e:
        # Log the error and show a notification
        xbmc.log(f"Error: {str(e)}", xbmc.LOGERROR)
        xbmcgui.Dialog().notification("Webshare", f"Playback failed: {str(e)}", xbmcgui.NOTIFICATION_ERROR)
        # Signal failure
        xbmcplugin.setResolvedUrl(handle, False, xbmcgui.ListItem())

def router(paramstring):
    log(f"Router invoked with paramstring: {paramstring}")

    params = dict(urllib.parse.parse_qsl(paramstring))
    if 'query' in params:
        list_search_results(params['query'])
    elif 'action' in params and params['action'] == 'play' and 'id' in params:
        try:
            play_video(handle, params)
        except Exception as e:
            xbmcgui.Dialog().notification("Webshare", str(e), xbmcgui.NOTIFICATION_ERROR)
            xbmcplugin.setResolvedUrl(handle, False, xbmcgui.ListItem())
    elif 'action' in params:
        if params['action'] == 'search':
            keyboard = xbmcgui.Dialog().input("Enter search query", type=xbmcgui.INPUT_ALPHANUM)
            if keyboard:
                list_search_results(keyboard)
            else:
                xbmcplugin.endOfDirectory(handle)
        elif params['action'] == 'about':
            xbmcgui.Dialog().ok("About Webshare", "This addon allows searching Webshare.cz files.")
            xbmcplugin.endOfDirectory(handle)
        elif params['action']  == 'series':
            list_series_results()

        elif params['action']  == 'list_seasons' and 'serie' in params:
            serie = params['serie']
            list_seasons(serie)

        elif params['action']  == 'list_episodes' and 'serie' in params and 'season' in params:
            serie = params['serie']
            season = params['season']
            list_episodes(serie, season)

        elif params['action']  == 'show_episode' and 'serie' in params and 'season' in params and 'episode' in params:
            serie = params['serie']
            season = params['season']
            episode = params['episode']
            show_episode(serie, season, episode)

        elif params['action'] == 'login':
            prompt_login()
            xbmcplugin.endOfDirectory(handle)
        else:
            xbmcplugin.endOfDirectory(handle)

    else:
        url_search = f"{sys.argv[0]}?action=search"
        item_search = xbmcgui.ListItem(label="Search")
        xbmcplugin.addDirectoryItem(handle=handle, url=url_search, listitem=item_search, isFolder=True)

        url_search = f"{sys.argv[0]}?action=series"
        item_search = xbmcgui.ListItem(label="Series")
        xbmcplugin.addDirectoryItem(handle=handle, url=url_search, listitem=item_search, isFolder=True)

        url_login = f"{sys.argv[0]}?action=login"
        item_login = xbmcgui.ListItem(label="Login")
        xbmcplugin.addDirectoryItem(handle=handle, url=url_login, listitem=item_login, isFolder=False)

        url_about = f"{sys.argv[0]}?action=about"
        item_about = xbmcgui.ListItem(label="About")
        xbmcplugin.addDirectoryItem(handle=handle, url=url_about, listitem=item_about, isFolder=False)

        xbmcplugin.endOfDirectory(handle)




if __name__ == '__main__':
    router(sys.argv[2][1:])
